/***********************************************************************/
/*                                                                     */
/*                              Pio (C++)                              */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*                  projet Estime, INRIA-Rocquencourt                  */
/*                                                                     */
/*  Copyright 2004-2011 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/********************************************************************************/
/*                                                                              */
/*                                  Pio (C++)                                   */
/*                                                                              */
/*                        Polyglot Input/Output Library                         */
/*                                                                              */
/*                      projet Estime, INRIA-Rocquencourt                       */
/*                                                                              */
/*  Copyright 2004-2011 INRIA.                                                  */
/*  Institut National de Recherche en Informatique et en Automatique.           */
/*  All rights reserved. This file is distributed only by permission.           */
/*                                                                              */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                                */
/*                                                                              */
/********************************************************************************/

/***********************************************************************/
/*                                                                     */
/*                              PIO (C++)                              */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*                  projet Estime, INRIA-Rocquencourt                  */
/*                                                                     */
/*  Copyright 2005-2007 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/* $Id: receive.cpp,v 1.3 2011-02-23 17:43:29 fclement Exp $ */

#include "boolean.hpp"
#include "receive.hpp"

//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////

using namespace std;
namespace io{
  
  
  //defined in single_types.cpp:
  void set_reception_binary();
  void set_reception_ascii();
  
  
  template<class dataType> void syntaxCheck(istream& ib,string f,dataType c1,dataType c2){
    if(c1 != c2){
      if(ib.eof()){
        EndOfStream k;
        throw(k);
      }
      cerr<<"syntax error in "<<f<<": \""<<c1<<"\" expected, \""<<c2<<"\" found."<<endl;
      SyntaxError t;throw(t);
    }
  }
  
  void r_check(istream& ib,string f,char c){
    char temp;
    ib>>temp;
    syntaxCheck(ib,f,c,temp);
  }
  
  void r_check(istream& ib,string f,string c){
    string temp;
    ib>>temp;
    syntaxCheck(ib,f,c,temp);
  }
  
  void r_check_line(istream& ib,string f,string c){
    string temp;
    getline(ib,temp,'\n');
    syntaxCheck(ib,f,c,temp);
  }
  
  void r_begin(istream& ib,string f1){
    r_check(ib,f1,"begin");
    char t[10];ib.getline(t,10,'\n');//r_check_line not used to enable to make follow several communications in the same stream.
  }
  
  void r_end(istream& ib,string f1){
    r_check(ib,f1,"end");
  }
  
  void r_newline(istream& ib, string f1){
    char c = ib.get();
    syntaxCheck(ib, f1, '\n', c);
  }
  
  void just_receive_double(istream& ib,double& it){
    if(get_reception_binary()){
      r_check(ib,"just_receive_double", '&');
      r_check(ib,"just_receive_double", '<');
      r_check(ib,"just_receive_double", '8');
      r_check(ib,"just_receive_double", '>');
      char eightchar[ sizeof(double) ];
      for(int i=0;i<8;i++)
        ib.get(eightchar[i]);
      it = reinterpret_cast<double&> (eightchar);
    }
    else
      ib>>it;
  }
  
  bool is_integer(char c){
    return (c >= '0' && c < '0'+10);
  }
  
  void receive_scalar(istream& ib,string& s){//string
    unsigned int string_size;
    string f1 = "receive_string input: ";
    r_check(ib,f1,'<');
    ib>>string_size;
    r_check(ib,f1,'>');
    r_check(ib,f1,'\"');
    getline(ib,s,'\"');
    for(string::size_type ii = s.find('\\',0);
	ii != string::npos; 
	ii = s.find('\\',ii+1)){
      if(s[ii+1] == 'b'){
	s.replace(ii,2,1,8);
      }
      else if(s[ii+1] == 'n'){
	s.replace(ii,2,1,10);
      }
      else if(s[ii+1] == 'r'){
	s.replace(ii,2,1,13);
      }
      else if(s[ii+1] == 't'){
	s.replace(ii,2,1,9);
      }
      else if(s[ii+1] == '\\'){
	s.replace(ii,2,1,92);
      }
      else if(s[ii+1] == '\"'){
	s.replace(ii,2,1,34);
      }
      else if(s[ii+1] == '\''){
	s.replace(ii,2,1,39);
      }
      else if(s[ii+1] == ' '){
	s.replace(ii,2,1,32);
      }
      else{
	if( is_integer(s[ii+1]) &&
	    is_integer(s[ii+2]) &&
	    is_integer(s[ii+3]) ){
	  const char c = (s[ii+1]-'0')*100+(s[ii+2]-'0')*10+(s[ii+3]-'0');
	  s.replace(ii,4,1,c);
	}
	else{
	  std::cerr<<"Invalid escape character "
		   <<s[ii+1]<<s[ii+2]<<s[ii+3]<<std::endl;
	  exit(1);
	}
      }
    }
    if(s.size() != string_size){
      cerr<<"String \""<<s<<"\" has length "<<s.size()
	  <<" but should have length "<<string_size<<" !"<<endl;
      StringLengthError t;throw(t);
    }
  }
  
  template<class dataType,int n> void  receive_scalar(istream& ib,Tuple<dataType,n>& v){//tuple
    string f1="input of receive_tuple_item: ";
    r_check(ib,f1,'(');
    for(int i=0;i<n-1;i++){
      ib>>v[i];
      //      cerr<<"v[i]="<<v[i]<<"\n";
      r_check(ib,f1,',');
      char c=ib.get();
      syntaxCheck(ib,f1,' ',c);
    }
    ib>>v[n-1];
    //    cerr<<"v[n-1]="<<v[n-1]<<"\n";
    r_check(ib,f1,')');
    //    cerr<<")\n";
  }
  
  
  template<int n> void  receive_scalar(istream& ib,Tuple<double,n>& v){//tuple
    string f1="input of receive_double_tuple_item: ";
    r_check(ib,f1,'(');
    for(int i=0;i<n-1;i++){
      just_receive_double(ib,v[i]);
      r_check(ib,f1,',');
      char c=ib.get();
      syntaxCheck(ib,f1,' ',c);
    }
    just_receive_double(ib,v[n-1]);
    r_check(ib,f1,')');
  }
  
  
  
  template<class dataType1,class dataType2> void  receive_scalar(istream& ib,pair<dataType1,dataType2>& v){//pair
    string f1="input of receive_pair_item: ";
    r_check(ib,f1,'(');
    ib>>v.first;
    r_check(ib,f1,',');
    char c=ib.get();
    syntaxCheck(ib,f1,' ',c);
    ib>>v.second;
    r_check(ib,f1,')');
  }
  
  
  template<class dataType2> void  receive_scalar(istream& ib,pair<double,dataType2>& v){//pair
    string f1="input of receive_double1_pair_item: ";
    r_check(ib,f1,'(');
    just_receive_double(ib,v.first);
    r_check(ib,f1,',');
    char c=ib.get();
    syntaxCheck(ib,f1,' ',c);
    ib>>v.second;
    r_check(ib,f1,')');
  }
  
  template<class dataType1> void  receive_scalar(istream& ib,pair<dataType1,double>& v){//pair
    string f1="input of receive_double2_pair_item: ";
    r_check(ib,f1,'(');
    ib>>v.first;
    r_check(ib,f1,',');
    char c=ib.get();
    syntaxCheck(ib,f1,' ',c);
    just_receive_double(ib,v.second);
    r_check(ib,f1,')');
  }
  
  
  void receive_scalar(istream& ib,double& it){
    just_receive_double(ib,it);
  }
  
  
  template<class typeName> void receive_scalar(istream& ib,typeName& it){//other scalar
    ib>>it;
  }
  
  
  template<class typeName> void check_vector_size(unsigned int n,vector<typeName>& v){
    if(v.size() != n){
      v.resize(n);
    }
  }
  
  template<class typeName> void receive_vector_items(istream& ib,unsigned int n, vector<typeName>& v){
    check_vector_size(n,v);
    unsigned int i=0;
    for(i=0;i<n;i++){
      receive_scalar(ib,v[i]);
      r_check(ib,"input of  receive_vector_items", ';');
    }
  }
  
  
  void receive_vector_items(istream& ib,unsigned int n, vector<double>& v){
    check_vector_size(n,v);
    //    r_newline(ib, "input of receive_vector_items (double)");
    for(unsigned int i=0;i<n;i++){
      receive_scalar(ib,v[i]);
      r_check(ib,"input of  receive_vector_items (double)", ';');
      r_newline(ib, "input of receive_vector_items (double)");
    }
  }
  
  
  template<class dataType> void receive_value_type(istream& ib,dataType i){
    string f1 = "input of receive_value_type";
    string temp;
    getline(ib,temp,'\n');
    if(temp.find("%bf",0)!=string::npos) set_reception_binary();
    else if (temp.find("%f",0)!=string::npos) set_reception_ascii();
    string s=type(i,RECEIVE);
    syntaxCheck(ib,f1,s,temp);
  }
  
  
  template<class typeName> void receive_value_type(istream& ib,vector<typeName>& t){
    string f1="input of receive_value_type (vector)";
    typeName tn;
    string temp;
    getline(ib,temp,'\n');
    if (temp.find("%bf",0)!=string::npos) set_reception_binary();
    else if( temp.find("%f",0)!=string::npos) set_reception_ascii();
    string fmt="[1"+type(tn,RECEIVE)+"1]";
    syntaxCheck(ib,f1,fmt,temp);
  }
  
  
  template<class typeName> void init_receive_vector(istream& ib,unsigned int& n, vector<typeName>& t){
    string f1="input of init_receive_vector";
    r_check(ib,f1,'<');
    ib>>n;
    r_check(ib,f1,'>');
    r_newline(ib, f1);
    check_vector_size(n,t);
  }
  
  template<class typeName> void receive_vector(istream& ib, vector<typeName>& t){
    unsigned int n;
    init_receive_vector(ib,n,t);
    receive_vector_items(ib,n,t);
  }
  
  template<class typeName> void receive_value(istream& ib,vector<typeName>& result ){
    string f1 = "receive_value vector input";
    r_check(ib,f1,"[1");
    receive_vector(ib,result);
    r_check(ib,f1,"1];");
  }
  
  template<class typeName>
  void receive_scalar_item(istream& ib, typeName& t){
    string f1 = "receive_scalar_item";
    receive_scalar(ib, t);
    r_check(ib, f1, ';');
    r_newline(ib, f1);
  }
  
  template<class typeName> void receive_value(istream& ib,typeName& t){
    string f1 = "receive_value scalar input";
    receive_scalar_item(ib,t);
  }
  
  template<class dataType> void receive(istream& ib,dataType& result){
    string f1 = "receive";
    r_begin(ib,f1);
    receive_value_type(ib,result);
    receive_value(ib,result);
    r_end(ib,f1);
  }
  
  //functions declared in the header file:
  void receive_boolean(istream& ib,boolean& b){receive(ib,b);}
  void receive_string(istream& ib,string& s){receive(ib,s);}
  void receive_int(istream& ib,int& i){receive(ib,i);}
  void receive_int_couple(istream& ib,int_couple& i_c){receive(ib,i_c);} 
  void receive_int_triple(istream& ib,int_triple& i_c){receive(ib,i_c);} 
  void receive_float(istream& ib,double& d){receive(ib,d);}
  void receive_float_couple(istream& ib,float_couple& p){receive(ib,p);}
  void receive_float_triple(istream& ib,float_triple& p){receive(ib,p);}
  void receive_boolean_couple(istream& ib,boolean_couple& p){receive(ib,p);}
  void receive_boolean_triple(istream& ib,boolean_triple& p){receive(ib,p);}
  void receive_boolean_vector(istream& ib,vector<boolean>& b){
    receive(ib,b);
    // string f1 = "receive";
    // r_begin(ib,f1);
    // receive_value_type(ib,b);
    // receive_value(ib,b);
    // r_end(ib,f1);
  }
  void receive_boolean_couple_vector(istream& ib,vector<boolean_couple>& v){receive(ib,v);}
  void receive_boolean_triple_vector(istream& ib,vector<boolean_triple>& v){receive(ib,v);}
  void receive_string_vector(istream& ib,vector<string>& s){receive(ib,s);}
  void receive_int_vector(istream& ib,vector<int>& v){receive(ib,v);}
  void receive_float_vector(istream& ib,vector<double>& v){receive(ib,v);}
  void receive_float_couple_vector(istream& ib,vector<float_couple>& v){receive(ib,v);}
  void receive_float_triple_vector(istream& ib,vector<float_triple>& v){receive(ib,v);}
  void receive_int_couple_vector(istream& ib,vector<int_couple>& v){receive(ib,v);}
  void receive_int_triple_vector(istream& ib,vector<int_triple>& v){receive(ib,v);}
  void receive_int_float_couple(istream& ib,int_float_couple& f){receive(ib,f);}
  void receive_float_int_couple(istream& ib,float_int_couple& f){receive(ib,f);}
  void receive_int_string_couple(istream& ib,int_string_couple& f){receive(ib,f);}
  void receive_string_int_couple(istream& ib,string_int_couple& f){receive(ib,f);}
  void receive_string_float_couple(istream& ib,string_float_couple& f){receive(ib,f);}
  void receive_float_string_couple(istream& ib,float_string_couple& f){receive(ib,f);}
  //
  boolean receive_boolean(istream& ib){boolean b;receive(ib,b);return (boolean)b;}
  boolean_couple receive_boolean_couple(istream& ib){boolean_couple b_c;receive(ib,b_c);return b_c;}
  boolean_triple receive_boolean_triple(istream& ib){boolean_triple b_c;receive(ib,b_c);return b_c;}
  string receive_string(istream& ib){string s;receive(ib,s);return s;}
  int receive_int(istream& ib){int i;receive(ib,i);return i;}
  int_couple receive_int_couple(istream& ib){int_couple i_c;receive(ib,i_c);return i_c;} 
  int_triple receive_int_triple(istream& ib){int_triple i_t;receive(ib,i_t);return i_t;} 
  double receive_float(istream& ib){double d;receive(ib,d);return d;}
  float_couple receive_float_couple(istream& ib){float_couple f_c;receive(ib,f_c);return f_c;}
  float_triple receive_float_triple(istream& ib){float_triple f_t;receive(ib,f_t);return f_t;}
  vector<boolean> receive_boolean_vector(istream& ib){vector<boolean> v_t;receive(ib,v_t);return v_t;}
  vector<boolean_couple> receive_boolean_couple_vector(istream& ib){vector<boolean_couple> v;receive(ib,v);return v;}
  vector<boolean_triple> receive_boolean_triple_vector(istream& ib){vector<boolean_triple> v;receive(ib,v);return v;}
  vector<string> receive_string_vector(istream& ib){vector<string> v_t;receive(ib,v_t);return v_t;}
  vector<int> receive_int_vector(istream& ib){vector<int> v_i;receive(ib,v_i);return v_i;}
  vector<double> receive_float_vector(istream& ib){vector<double> v_d;receive(ib,v_d);return v_d;}
  vector<float_couple> receive_float_couple_vector(istream& ib){vector<float_couple> v;receive(ib,v);return v;}
  vector<float_triple> receive_float_triple_vector(istream& ib){vector<float_triple> v;receive(ib,v);return v;}
  vector<int_couple> receive_int_couple_vector(istream& ib){vector<int_couple> v;receive(ib,v);return v;}
  vector<int_triple> receive_int_triple_vector(istream& ib){vector<int_triple> v;receive(ib,v);return v;}
  //
  //|---------------------|
  //|for matrix reception:|
  //|---------------------|
  
  
  //The two following functions read the part of the stream 'ib' which should be on the form:
  
  // "[2" Scalar_type "2]" .
  
  //The type of the matrix elements must be known before calling this function.
  //The coherence of the type of the matrix elements, imposed by 'typeName', and the Scalar_type, is checked. 
  
  //The type of the second argument corresponds to the type of storage chosen.
  
  template<class typeName> void receive_value_type(istream& ib,typeName* & t){
    string f1="input of receive_value_type (matrix *)";
    typeName tn;
    string temp;
    getline(ib,temp,'\n');
    if (temp.find("%bf")!=string::npos) set_reception_binary();
    else if( temp.find("%f")!=string::npos) set_reception_ascii();
    string fmt="[2"+type(tn,RECEIVE)+"2]";
    syntaxCheck(ib,f1,fmt,temp); 
  }
  
  
  template<class typeName> void receive_value_type(istream& ib,typeName** & t){
    string f1="input of receive_value_type (matrix **)";
    typeName tn;
    string temp;
    getline(ib,temp,'\n');
    if (temp.find("%bf")!=string::npos) set_reception_binary();
    else if (temp.find("%f")!=string::npos) set_reception_ascii();
    string fmt="[2"+type(tn,RECEIVE)+"2]";
    syntaxCheck(ib,f1,fmt,temp); 
  }
  
  //The two following functions read the part of the stream 'ib' which should be on the form:
  
  //"[2" '\n'
  //'<' Integer ", " Integer '>' '\n'
  // Array_layout
  
  //the dimensions of the matrix, which should be unknown before the call of the function, are stored in 'm' and 'n'.
  
  //In the first version, the layout must be known, given as an argument under the form of a string "row" or "column", and is checked.
  //In the second version, the layout is read and returned on the form of a string, "row" or "column".
  
  void init_receive_float_matrix(istream& ib,unsigned int& m,unsigned int& n,string layout){
    string f1 = "init_receive_float_matrix";
    char c;
    
    r_check(ib,f1,"[2");
    r_check(ib,f1,'<');
    ib>>m;
    r_check(ib,f1,',');
    c=ib.get();
 syntaxCheck(ib,f1,' ',c);
    ib>>n;
    r_check(ib,f1,'>');
    char tt;
    ib>>tt;
    if(layout=="row"){
      if(tt=='0'){
        r_check(ib,f1,',');
        c=ib.get();
        syntaxCheck(ib,f1,' ',c);
        r_check(ib,f1,'1');
      }
      else if(tt != 'C') {cerr<<"layout error"<<endl;exit(1);}
    }
    else if(layout=="column"){
      if(tt=='1'){
        r_check(ib,f1,',');
        c=ib.get();
        syntaxCheck(ib,f1,' ',c);
        r_check(ib,f1,'0');
      }
      else if(tt != 'F') {cerr<<"layout error"<<endl;exit(1);}
    }
    else{
      cerr<<"wrong layout"<<endl;exit(1);
    }
  }
  
  string init_receive_float_matrix(istream& ib,unsigned int& m,unsigned int& n){
    string f1 = "init_receive_float_matrix";
    char c;
    
    r_check(ib,f1,"[2");
    r_check(ib,f1,'<');
    ib>>m;
    r_check(ib,f1,',');
    c=ib.get();
    syntaxCheck(ib,f1,' ',c);
    ib>>n;
    r_check(ib,f1,'>');
    char tt;
    ib>>tt;
    string layout;
    if(tt=='0'){
      r_check(ib,f1,',');
      c=ib.get();
      syntaxCheck(ib,f1,' ',c);
      r_check(ib,f1,'1');
      layout = "row";
    }
    else if(tt == 'C') {layout = "row";}
    else if(tt=='1'){
      r_check(ib,f1,',');
      c=ib.get();
      syntaxCheck(ib,f1,' ',c);
      r_check(ib,f1,'0');
      layout = "column";
    }
    else if(tt == 'F') {layout = "column";}
    else{
      cerr<<"wrong layout"<<endl;exit(1);
    }
    return layout;
  }
  
  
  //the four following functions read the stream 'ib' from the key word 
  //begin
  //go as far as the two previous functions, checking all what is checked by the four previous functions,
  //and allocate memory to store the data according to the read numbers of rows and columns.
  //When we choose the storage on the form of **, this allocation is partial for the moment.
  //layout can be imposed or returned.
  
  
  void init_receive_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double*& M,string layout){
    string f1 = "init_receive_float_matrix";
    r_begin(ib,f1);
    receive_value_type(ib,M);
    init_receive_float_matrix(ib,m,n,layout);
    M = new double[m*n];
  }
  
  string init_receive_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double*& M){
    string f1 = "init_receive_float_matrix";
    r_begin(ib,f1);
    receive_value_type(ib,M);
    string layout=init_receive_float_matrix(ib,m,n);
    M = new double[m*n];
    return layout;
  }
  
  void init_receive_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double**& M,string layout){
    string f1 = "init_receive_float_matrix";
    r_begin(ib,f1);
    receive_value_type(ib,M);
    init_receive_float_matrix(ib,m,n,layout);
    M = new double*[m];
  }
  
  string init_receive_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double**& M){
    string f1 = "init_receive_float_matrix";
    r_begin(ib,f1);
    receive_value_type(ib,M);
    string layout = init_receive_float_matrix(ib,m,n);
    M = new double*[m];
    return layout;
  }
  
  //function to read and check
  // "2]"
  // end
  // :
  void end_receive_float_matrix(istream& ib){
    string f1 = "end_receive_float_matrix";
    r_check(ib,f1,"2];");
    r_end(ib,f1);
  }
  
  //the two following functions read the succession of Lines of a Matrix input which is stores in a simple array. The meaning of the arguments 'm' and 'n' (known before the call of the function) differ upon the layout:
  // for "receive_loop_layout_same_layout" :
  // stream  | storage  | m number of: |  n number of:
  // C       | C        | lines        | columns
  // F       | F        | columns      | lines
  // for "receive_loop_layout_other_layout" :
  // stream  | storage  | m number of: |  n number of:
  // F       | C        | lines        | columns
  // C       | F        | columns      | lines
  void receive_loop_layout_same_layout(istream& ib,unsigned int& m,unsigned int& n,double* M,string f1){
    double* MM;
    MM=M;
    char c;
    for(unsigned int mm=0;mm<m;mm++){
      r_check(ib,f1,"[|");
      c=ib.get();
      syntaxCheck(ib,"receive_loop_layout_same_layout:",'\n',c);
      for(unsigned int nn=0;nn<n;nn++){
        just_receive_double(ib,*MM);
        MM++;
        r_check(ib,f1,';');
        c=ib.get();
        syntaxCheck(ib,"receive_loop_layout_same_layout:",'\n',c);
      }
      r_check(ib,f1,"|];");
    }
  }
  
  void receive_loop_layout_other_layout(istream& ib,unsigned int& m,unsigned int& n,double* M,string f1){
    char c;
    for(unsigned int nn=0;nn<n;nn++){
      r_check(ib,f1,"[|");
      c=ib.get();
      syntaxCheck(ib,"receive_loop_layout_other_layout:",'\n',c);
      for(unsigned int mm=0;mm<m;mm++){
        just_receive_double(ib,M[n*mm+nn]);
        r_check(ib,f1,';');
        c=ib.get();
        syntaxCheck(ib,"receive_loop_layout_other_layout:",'\n',c);
      }
      r_check(ib,f1,"|];");
    }
  }
  
  //this function reads a matrix stream from the beginning of matrix string "[2" to the end of the end of matrix string "2]", using same layout for storage as for stream. Layout is imposed, no default value. Simple array form:
  void receive_value_same_layout(istream& ib,unsigned int& m,unsigned int& n,double*& M,string layout){
    init_receive_float_matrix(ib,m,n,layout);//without arg M: begin and type not read.
    string f1 = "receive_value_same_layout (matrix *)";
    M=new double[m*n];
    if(layout == "row")
      receive_loop_layout_same_layout(ib,m,n,M,f1);
    else
      receive_loop_layout_same_layout(ib,n,m,M,f1);
    r_check(ib,f1,"2];");
  }
  
  
  //this function reads a matrix stream from the beginning of matrix string "[2" to the end of the end of matrix string "2]", using same layout for storage as for stream. Layout is unknown and returned. Simple array form:
  string receive_value_same_layout(istream& ib,unsigned int& m,unsigned int& n,double*& M){
    string layout=init_receive_float_matrix(ib,m,n);//without arg M: begin and type not read.
    string f1 = "receive_value_same_layout (matrix *)";
    M=new double[m*n];
    if(layout == "row")
      receive_loop_layout_same_layout(ib,m,n,M,f1);
    else
      receive_loop_layout_same_layout(ib,n,m,M,f1);
    r_check(ib,f1,"2];");
    return layout;
  }
  
  
  //complete reception and checking from a C stream to a C storage (simple array).
  void receive_row_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double*& M){
    init_receive_float_matrix(ib,m,n,M,"row");
    string f1 = "receive_row_float_matrix";
    receive_loop_layout_same_layout(ib,m,n,M,f1);
    end_receive_float_matrix(ib);
  }
  
  //this function reads a matrix stream from the beginning of matrix string "[2" to the end of the end of matrix string "2]". Layout is imposed, default value is "row", there is no layout for the multiple array storage. multiple array form:
  void receive_value(istream& ib,unsigned int& m,unsigned int& n,double** & M,string layout="row"){
    init_receive_float_matrix(ib,m,n,layout);//without arg M: begin and type not read.
    string f1 = "receive_value_same_layout (matrix **)";
    M=new double*[m];
    double** MM=M;
    char c;
    if(layout != "row"){cerr<<"case not implemented yet"<<endl;exit(-1);}
    for(unsigned int mm=0;mm<m;mm++){
      r_check(ib,f1,"[|");
      c=ib.get();
      syntaxCheck(ib,f1,'\n',c);
      *MM=new double[n];
      double* K=*MM;
      for(unsigned int nn=0;nn<n;nn++){
        just_receive_double(ib,*K);
        K++;
        r_check(ib,f1,';');
        c=ib.get();
      syntaxCheck(ib,f1,'\n',c);
      }
      MM++;
      r_check(ib,f1,"|];");
    }
    r_check(ib,f1,"2];");
  }

  //complete reception and checking from a C stream (multiple array):
  void receive_row_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double** & M){
    string f1 = "receive_row_float_matrix";
    r_begin(ib,f1);
    receive_value_type(ib,M);
    receive_value(ib,m,n,M);
    r_end(ib,f1);
  }
  
  //complete reception and checking from a F stream to a C storage (simple array):
  void receive_column_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double*& M){
    init_receive_float_matrix(ib,m,n,M,"column");
    string f1 = "receive_column_float_matrix";
    receive_loop_layout_other_layout(ib,m,n,M,f1);
    end_receive_float_matrix(ib);
  }

  //complete reception and checking from a F stream (multiple array):
  void receive_column_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double** & M){
    init_receive_float_matrix(ib,m,n,M,"column");
    string f1 = "receive_column_float_matrix";
    double** MM=M;
    for(unsigned int mm=0;mm<m;mm++){
      *MM=new double[n];
      MM++;
    }
    char c;
    for(unsigned int nn=0;nn<n;nn++){
      r_check(ib,f1,"[|");
      c=ib.get();
      syntaxCheck(ib,f1,'\n',c);
      for(unsigned int mm=0;mm<m;mm++){
        ib>>M[mm][nn];
        r_check(ib,f1,';');
        c=ib.get();
      syntaxCheck(ib,f1,'\n',c);
      }
      r_check(ib,f1,"|];");
    }
    end_receive_float_matrix(ib);
  }

  //complete reception and checking from a C stream to a C storage (simple array):
  void receive_row_from_row_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double* & M){
    receive_row_float_matrix(ib,m,n,M);
  }
  
  //complete reception and checking from a F stream to a C storage (simple array):
  void receive_row_from_column_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double* & M){
    receive_column_float_matrix(ib,m,n,M);
  }
  
  //complete reception and checking from a C stream to a F storage (simple array):
  void receive_column_from_row_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double* & M){
    init_receive_float_matrix(ib,m,n,M,"row");
    string f1 = "receive_column_from_row_float_matrix";
    receive_loop_layout_other_layout(ib,n,m,M,f1);
    end_receive_float_matrix(ib);
  }

  //complete reception and checking from a F stream to a F storage (simple array):
  void receive_column_from_column_float_matrix(istream& ib,unsigned int& m,unsigned int& n,double* & M){
    init_receive_float_matrix(ib,m,n,M,"column");
    string f1 = "receive_column_from_column_float_matrix";
    receive_loop_layout_same_layout(ib,n,m,M,f1);
    end_receive_float_matrix(ib);
   }


  //to receive a vector one-line matrix(C stream to C storage)(simple array): 
  void receive_float_array(istream& ib,unsigned int& n,double* & M){
    unsigned int m;
    receive_row_float_matrix(ib,m,n,M);
    if(m != 1) {cerr<<"wrong first dimension "<<m<<" in receive_float_array."<<endl;exit(1);}
  }

}
