(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2004-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: communication.mli,v 1.2 2011-02-23 17:43:29 fclement Exp $ *)

(** {3 The definition of communications} *)

(** Communications are the elementary information units that are exchanged
    within the [Io] library: each read or write operation has such an entity as
    argument. *)

type values = Typed_value.t array;;

type service_name =
   | Ok | Ko | Allo | Bye | Start | Stop | WrongCommunication
;;

type task_name = string;;

type t =
   | Phrase of values
   | Task of task_name * values
   | Result of values
   | Error of values
   | Service of service_name * values
(** The general type for communications. *)
;;

val service_name_of_string : string -> service_name;;
val string_of_service_name : service_name -> string;;
(** Convertions from/to [string]. *)
